#!/bin/env perl 
#!d:\perl\bin\perl.exe 

# -- Copyright (C) 2001 Paul Kulchenko --

use strict;
use SOAP::Lite;
use File::Path;

print "\n\nWelcome to CodeShare! The Open source code sharing network!";
print "\nCopyright(c) 2001, James Snell, Paul Kulchenko, Doug Tidwell\n\n";

@ARGV or die "Usage: $0 CodeShareServer [commands...] [-dump [filename]] \n";
my $proxy = shift;
my $uri = 'http://namespaces.soaplite.com/CodeShare/Owner';
my $soap = SOAP::Lite->proxy($proxy)->uri($uri)->on_fault(sub{});

my($dump, $file) = @ARGV > 0 && @ARGV[-1] eq '-dump' ? splice(@ARGV, -1, 1) :
                   @ARGV > 1 && @ARGV[-2] eq '-dump' ? splice(@ARGV, -2, 2) :
                   (undef, undef);
if ($dump) {
  print STDERR "Wiredumps are logged in '$file'\n" if $file;
  $file ||= '&STDOUT';                     # STDOUT by default
  open(F, ">>$file") or die "$file: $!\n"; # open in append mode
  select((select(F), $|=1)[0]);            # select non-buffered output
  $soap->on_debug(sub{print F @_});        # debug goes there
  eval "END { close F }";                  # close handle when we are done
}

print STDERR "Usage: { search | info | get | list | quit | help } [parameters...]\n> ";

while (defined($_ = shift || <>)) {
  next unless /\w/;
  my($method, $modifier, $parameters) = m!^\s*(\w+)(?:\s*/(\w*)\s)?\s*(.*)!;

  last if $method =~ /^q(?:uit)?$/i;
  help(), next if $method =~ /^h(?:elp)?$/i;

  my $res = eval "\$soap->$method('$parameters', '$modifier')";

  # check for errors
  $@                            and print(STDERR join "\n", $@, ''), next;
  defined($res) && $res->fault  and print(STDERR join "\n", $res->faultstring, ''), next;
  !$soap->transport->is_success and print(STDERR join "\n", $soap->transport->status, ''), next;

  # check for result
  my @result = @{$res->result} or print(STDERR "No matches\n"), next;

  foreach (@result) {
    print(STDERR "$_->{type}: @{[join ', ', $_->{Title} || (), $_->{path} || ()]}\n");
    if ($method eq 'get') {
      if ($_->{type} eq 'directory') { File::Path::mkpath($_->{path}) }
      if ($_->{type} eq 'file') { 
        open(F, '>'. $_->{path}) or warn "$_->{path}: $!\n";
        print F $_->{file};
        close(F) or warn "$_->{path}: $!\n";
      }
    } elsif ($method eq 'info') {
      foreach my $key (grep {$_ !~ /^(?:type|path)/} keys %$_) {
        print "  $key: $_->{$key}\n";
      }
    }
  }
} continue {
  print STDERR "\n> ";
}

sub help {
  print "Short help about search, info, get and list commands is here\n";
}
