# ======================================================================
# Copyright (C) 2001 Paul Kulchenko (paulclinger@yahoo.com)
# ======================================================================

package CodeShare::Owner;

use strict;

my $index;                                      # parsed index file
my $DC_NS = "http://purl.org/dc/elements/1.1/"; # Dublin Code namespace
my @ELEMENTS = qw(Title Creator Date Subject Description);

sub init {
  my($class, $root) = @_;
  open(F, $root) or die "$root: $!\n";
  $index = SOAP::Custom::XML::Deserializer->deserialize(join '', <F>)->root;
  close(F) or die "$root: $!\n";
}

sub traverse {
  my($self, %params) = @_;

  my $start = $params{start};

  my $type = $start->SOAP::Data::name; # file|project|directory
  my $location = ref $start->location ? $start->location->value : '';

  # path to current structure. Empty for projects
  my $path = $type eq 'directory' || 
             $type eq 'file' ? join('/', $params{path} || (), $location) : '';
  my $prefix = $type eq 'project' ? $location : $params{prefix} || '';
  my $fullpath = join '/', $prefix, $path; # full path. Used to GET files

  my $where = $params{where};
  my $matched = 
    $params{get} && $params{matched} ||
    $params{what} && 
    # check only subelements in Dublin Core namespace
    $start->$where() =~ /$params{what}/ && $start->$where()->uri eq $DC_NS;

  return 
    # current element
    ($matched
       ? +{ type => $type, 
            path => $path,
            ($params{get} ? (fullpath => $fullpath) : ()),
            map { ref $start->$_() ? ($_ => $start->$_()->value) : () 
                } @ELEMENTS
         } 
       : ()
    ), 

    # and everything below
    map { $self->traverse(start => $_, where => $where, what => $params{what}, 
                          path => $path, prefix => $prefix, 
                          get => ($params{get} || 0), matched => $matched) } 
      $start->project, $start->directory, ($type eq 'file' ? () : $start->file)
  ;
}

sub list {

  print("\nHandling a list request...");

  my($self, $what) = @_;
  [ map { my $e = $_; +{ map {$_ => $e->{$_}} qw(type path Title file fullpath) } }
      $self->traverse(start => $index, where => 'Title', what => $what, get => 1)
  ];
}

sub get {

  print("\nHandling a get request...");

  my $results = shift->list(@_);
  [ map { $_->{type} eq 'file' && open(F, delete $_->{fullpath}) 
            ? ($_->{file} = join('', <F>), close F) : (); $_ }
      @$results
  ];
}

sub search { # same as info(), but returns only 'type', 'path' and 'Title'

  print("\nHandling a search request...");

  my $results = shift->info(@_);
  [ map { my $e = $_; +{ map {$_ => $e->{$_}} qw(type path Title) } } 
      @$results 
  ];
}

sub info {

  print("\nHandling an info request...");

  my($self, $what, $where) = @_;
  [ $self->traverse(start => $index, 
                    where => $where || 'Title', what => $what || '.')
  ];
}

1;
