package codeshare.index;

import java.util.List;
import java.util.Vector;
import java.util.Iterator;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;

/**
 * @author James Snell (jasnell@us.ibm.com)
 */

public class DirectoryItem extends Item {

    protected List items;
    protected ProjectItem project;
   
    public void addItem(Item item) {
        if (item instanceof ProjectItem || item instanceof Index)
            throw new IllegalArgumentException("Projects and Indexes cannot be contained with DirectoryItems");
        if (items == null) items = new Vector();
        items.add(item);
        item.indexdc(project.index.indexes, project);
    }
    
    public Item getItem(int n) {
        if (items == null) return null;
        return (Item)items.get(n);
    }
    
    public List getItems() {
        return items;
    }
    
    public void serialize(Element parent) {
        Document doc = parent.getOwnerDocument();
        Element e = doc.createElement("directory");
        serializedc(e);
        serializeitems(e);
        parent.appendChild(e);
    }
    
    protected void serializeitems(Element parent) {
        if (items == null) return;
        Document doc = parent.getOwnerDocument();
        for (Iterator i = getItems().iterator(); i.hasNext();) {
            Item item = (Item)i.next();
            item.serialize(parent);
        }
    }
    
    public void init(Element source) {
        initdc(source);
        inititems(source);
    }
    
    protected void inititems(Element source) {
        NodeList nl = source.getChildNodes();
        for (int n = 0; n < nl.getLength(); n++) {
            Node nd = nl.item(n);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element e = (Element)nd;
                if (!dcns.equals(e.getNamespaceURI())) {
                    if ("project".equals(e.getNodeName())) {
                        ProjectItem item = new ProjectItem();
                        addItem(item);
                        item.init(e);
                        item.indexdc(project.index.indexes, project);
                    }
                    if ("directory".equals(e.getNodeName())) {
                        DirectoryItem item = new DirectoryItem();
                        addItem(item);
                        item.init(e);
                        item.indexdc(project.index.indexes, project);
                    }
                    if ("file".equals(e.getNodeName())) {
                        FileItem item = new FileItem();
                        addItem(item);
                        item.init(e);
                        item.indexdc(project.index.indexes, project);
                    }
                }
            }
        }
    }
    
    public String toString() {
        StringBuffer strbuf = new StringBuffer();
        strbuf.append("\nDirectory");
        toString(strbuf);
        return strbuf.toString();
    }
}
