package codeshare.index;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import codeshare.index.util.SimpleTextIndexes;

/**
 * @author James Snell (jasnell@us.ibm.com)
 */

public abstract class Item {

    protected static final String dcns = "http://purl.org/dc/elements/1.1/";
    
    protected Map dc;
    protected String location;
    
    public String getLocation() {
        return this.location;
    }
    
    public void setLocation(String location) {
        this.location = location;
    }
    
    public String getDublinCoreValue(String name) {
        if (dc == null) return null;
        return (String)dc.get(name);
    }
    
    public void setDublinCoreValue(String name, String value) {
        if (dc == null) dc = new HashMap();
        dc.put(name, value);
    }
    
    public Map getDublinCoreValues() {
        return this.dc;
    }
    
    public abstract void serialize(Element parent);
    
    public void init(Element source) {
        initdc(source);
    }
    
    protected void initdc(Element source) {
        String location = source.getAttribute("location");
        if (location != null) setLocation(location);
        NodeList nl = source.getChildNodes();
        for (int n = 0; n < nl.getLength(); n++) {
            Node nd = (Node)nl.item(n);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                if (dcns.equals(nd.getNamespaceURI())) {
                    String name = nd.getNodeName();
                    String value = nd.getFirstChild().getNodeValue();
                    setDublinCoreValue(name, value);
                }
            }
        }
    }
    
    protected void serializedc(Element parent) {
        if (location != null) 
            parent.setAttribute("location", location);
        if (dc == null) return;
        Document doc = parent.getOwnerDocument();
        for (Iterator i = getDublinCoreValues().keySet().iterator(); i.hasNext();) {
            String name = (String)i.next();
            Element e = doc.createElementNS(dcns, name);
            e.setPrefix("dc");
            e.appendChild(doc.createTextNode(getDublinCoreValue(name)));
            parent.appendChild(e);
        }
    }
    
    protected void indexdc(SimpleTextIndexes indexes, ProjectItem project) {
        if (dc == null) return;
        String Title = getDublinCoreValue("Title");
        String Creator = getDublinCoreValue("Creator");
        String Subject = getDublinCoreValue("Subject");
        String Descrip = getDublinCoreValue("Description");
        if (Title != null) indexes.index("Title", Title, project);
        if (Creator != null) indexes.index("Creator", Creator, project);
        if (Subject != null) indexes.index("Subject", Subject, project);
        if (Descrip != null) indexes.index("Description", Descrip, project);
    }
    
    public void toString(StringBuffer strbuf) {
        if (dc == null) return;
        if (location != null)
            strbuf.append("\n\t@Location=" + location);
        for (Iterator i = dc.keySet().iterator(); i.hasNext();) {
            String name = (String)i.next();
            String value = getDublinCoreValue(name);
            strbuf.append("\n\tdc:" + name + "=" + value);
        }
    }
}
