package codeshare.service;

import java.util.List;
import java.util.Iterator;
import java.util.ArrayList;
import codeshare.index.Index;
import codeshare.index.ProjectItem;
import codeshare.index.Item;
import codeshare.index.util.IndexReader;
import codeshare.index.util.IndexWriter;

/**
 * @author James Snell (jasnell@us.ibm.com)
 */

public class IndexService {
    
    protected Index index;
    
    public IndexService() {
        index = IndexReader.readIndex("CodeShareIndex.xml");
        if (index == null) {
            System.out.println("CodeShareIndex.xml not found, creating default empty CodeShareIndex.xml");
            index = new Index();
            save();
        }
    }
    
    public void save() {
        IndexWriter.writeIndex("CodeShareIndex.xml", index);
    }
        
    public Index getIndex() {
        return this.index;
    }
    
    public List search(String p1, String p2) {
        if (p1 == null) p1 = "Title";
        List projects = index.search(p1, p2);
        ArrayList array = new ArrayList();
        if (projects != null) {
            for (Iterator i = projects.iterator(); i.hasNext();) {
                ProjectItem project = (ProjectItem)i.next();
                ProjectItem p = new ProjectItem();
                p.setLocation(project.getLocation());
                p.setDublinCoreValue("Title", project.getDublinCoreValue("Title"));
                array.add(p);
            }
        }
        return array;
    }
    
    public List get(String p1, String p2) {
        return info(p1,p2);
    }
    
    public List info(String p1, String p2) {
        List projects = index.getItems();
        ArrayList array = new ArrayList();
        if (projects != null){
            for (Iterator i = projects.iterator(); i.hasNext();) {
                ProjectItem project = (ProjectItem)i.next();
                if (p2 != null) {
                    String title = project.getDublinCoreValue("Title");
                    if (title != null) {
                        if (title.equals(p2)) {
                            array.add(project);
                        }
                    }
                } else {
                    array.add(project);
                }
            }
        }
        return array;
    }
    
    public List list(String p1, String p2) {
        List projects = index.getItems();
        ArrayList array = new ArrayList();
        if (projects != null){
            for (Iterator i = projects.iterator(); i.hasNext();) {
                ProjectItem project = (ProjectItem)i.next();
                if (p2 != null) {
                    String title = project.getDublinCoreValue("Title");
                    if (title != null) {
                        if (title.equals(p2)) {
                            ProjectItem p = new ProjectItem();
                            p.setLocation(project.getLocation());
                            p.setDublinCoreValue("Title", project.getDublinCoreValue("Title"));
                            array.add(p);
                        }
                    }
                } else {
                    ProjectItem p = new ProjectItem();
                    p.setLocation(project.getLocation());
                    p.setDublinCoreValue("Title", project.getDublinCoreValue("Title"));
                    array.add(p);
                }
            }
        }
        return array;
    }
    
}
